<%@ Page Language="C#" %>
<%@ Import Namespace="System.Data.SqlClient" %>
<%@ Import Namespace="System.Configuration" %>
<%@ Import Namespace="System.Collections" %>

<script runat="server">
    
protected void Page_Load(object sender, EventArgs e)
{
    // Set up SQL connection parameters
    SqlConnection sqlConn;
    SqlDataReader sqlDR;
    SqlCommand sqlCmd;

    // Set up SQL query parameters
    string strSQL;
    string strStartDate;
    string strEndDate;
    string strDate = Request["strDate"];
    string strAppName = Request["strAppName"];
    string strAppID = Request["strAppID"];

    string strRetXML; // Holds returned XML for the graph

    // array to hold sessions counts for each hour
    int[] hours = new int[24];

    // initialize all array values to 0
    for (int i = 0; i < 24; i++)
        hours[i] = 0;

    float utcOffset = float.Parse(Session["utcOffset"].ToString());

    strStartDate = strDate + " 0:0:0";
    strEndDate = strDate + " 23:59:59";

    // generate graph XML header data
    strRetXML = "<graph ";
    strRetXML += "basefont='Verdana' ";
    strRetXML += "bastfontsize='10px' ";
    strRetXML += "outcnvbasefontsize='10px' ";
    strRetXML += "caption='Number of sessions started on " + strDate + "' ";
    
    if (strAppName.Trim() != String.Empty)
        strRetXML += String.Format("subcaption='(for {0})' ", strAppName);
    
    strRetXML += "yaxisname='Sessions' ";
    strRetXML += "xaxisname='Time of Day' ";
    strRetXML += "decimalPrecision='0' ";
    strRetXML += "showGridBg='0' ";
    strRetXML += "showCanvas='1' ";
    strRetXML += "canvasbgcolor='ececec' ";
    strRetXML += "divlinecolor='ececec' ";
    strRetXML += "animation='0' ";

    // Set hover options
    strRetXML += "hovercapbg='f5f5f5' ";
    strRetXML += "hovercapborder='c0c0c0' ";
    strRetXML += "hovercapSepChar=' - logons:' ";
    
    strRetXML += "showhovercap='1' ";
    strRetXML += ">";

    // Done with header information, now loop through results
    // to generate individual hour details.

    // Connect to the RM summary database
    sqlConn = new SqlConnection(ConfigurationManager.ConnectionStrings["rmsummarydbConnectionString"].ConnectionString);
    sqlConn.Open();

    try
    {

        // Select sessions started on this day
        strSQL = String.Format("SELECT DATEADD(hh, {0}, SESSIONSTART) as sessionStarted FROM SDB_SESSION WHERE SESSIONSTART BETWEEN DATEADD(hh, {3}, '{1}') AND DATEADD(hh, {3}, '{2}')", utcOffset, strStartDate, strEndDate, utcOffset * -1);

        // add appid to the where clause if applicable
        if (strAppID.Trim() != String.Empty)
            strSQL += String.Format(" AND SDB_SESSION.FK_APPNAMEID = '{0}'", strAppID);
        
        // Execute the SQL command to retreive the data
        sqlCmd = new SqlCommand(strSQL, sqlConn);
        sqlDR = sqlCmd.ExecuteReader();

        while (sqlDR.Read())
        {
            DateTime sessionStarted = (DateTime)sqlDR["sessionStarted"];
            int hourStarted = sessionStarted.Hour;

            if (sessionStarted.ToShortDateString() == strDate)
            {
                hours[hourStarted]++;
            }
        }

        // Populate graph data set
        for (int x = 0; x < 24; x++)
        {
            string strName;
            string strShowName;
            string strColor = "5583D3";

            // Add labels to certain hours
            switch (x)
            {
                case 0:
                    strName = "Midnight";
                    break;
                case 12:
                    strName = "Noon";
                    break;
                default:
                    if (x > 12)
                        strName = (x - 12).ToString() + ":00 PM";
                    else
                        strName = x.ToString() + ":00 AM";
                    break;
            }

            if (x % 4 == 0)
                strShowName = "1";
            else
                strShowName = "0";

            strRetXML += String.Format("<set name='{0}' ", strName);
            strRetXML += String.Format("value='{0}' ", hours[x].ToString());
            strRetXML += String.Format("link='hourDetail.aspx?strDate={0}&strHour={1}&strAppID={2}'", strDate, x.ToString(), strAppID);
            strRetXML += String.Format(" showName='{0}' ", strShowName);
            strRetXML += "alpha='80' ";
            strRetXML += string.Format("color='{0}' ", strColor);
            strRetXML += " />";
        }

    }
    catch (Exception ex)
    {
        strRetXML += String.Format("<set name='Error: {0}' value='0'>", ex.Message);
    }
    finally
    {
        sqlConn.Close();
    }

    strRetXML += "</graph>";

    Response.Write(strRetXML);
}

</script>
